@@ -15,7 +15,7 @@ import numpy.typing as npt
 import pandas as pd
 import plotly.graph_objects as go
 import pysam
-from Bio import pairwise2
+
 from scipy import stats
 from tqdm.auto import tqdm
 
@@ -71,7 +71,7 @@ class BamOutput:
         """
 
         # Grab the promoter region of interest
-        samm: pysam.AlignmentFile = pysam.AlignmentFile(sams, "rb")
+        samm: pysam.AlignmentFile = pysam.AlignmentFile(str(sams), "rb")
         itern = samm.fetch(
             self._config.chromosome,
             int(self._config.promoter_start),
@@ -97,38 +97,22 @@ class BamOutput:
 
         return df2.index.unique().tolist()
 
-    def _write_bam_output(self, positions: List[str], df: pd.Series) -> None:
+    def _write_bam_output(self, positions: pd.Index, df: pd.Series) -> None:
         """Extract alignments from sequencing reads and output text strings
         in bam values dictionary.
 
         Args:
-            sams (Path): path to a sam file
-            positions (List[str]): list of unique positions
+            positions (pd.Index): list of unique positions
             df (pd.Series): series of sequencing reads
         """
 
         for each1 in positions:
-            alignments: List[str] = []
-
-            # Set up query using alignment algorithm
-            query_sequence: List[str] = df.loc[each1].head(1).tolist()[0]
-
-            # Set up sequences to check for alignment
-            target_sequences: List[str] = df.loc[each1].tolist()
-            for target_sequence in target_sequences:
-                alignment = pairwise2.align.localms(
-                    query_sequence, target_sequence, 2, -3, -5, -2
-                )
-                aligned_segment = alignment[0].seqA[
-                    alignment[0].start : alignment[0].end
-                ]
-                alignments.append(aligned_segment)
+            alignments: List[str] = df.loc[each1].tolist()
 
             read_file: List[str] = []
             for index, each in enumerate(alignments):
                 read_file.append(str(">read" + str(index)))
                 read_file.append(each)
-                # returns aligned target sequence
 
             self.values[each1] = "\n".join(read_file)
 
@@ -145,21 +129,26 @@ class BamOutput:
 
         Returns:
             str: genomic bases for indicated read / position
-        """
 
-        # OFFSET: int = 1298163  # TERT offset
+        Raises:
+            ValueError: incorrect position
+        """
 
         if not offset:
             offset = self._config.offset
 
-        start: int = offset - (int(position) + 30)
-        end: int = offset - (int(position) + 1)
+        start: int = int(position) - offset
+        end: int = (int(position) - offset) + 30
         if genome_string[start:end]:
             return genome_string[start:end]
         raise ValueError("Position not in genomic promoter file")
 
     def _genome_parsing(self, genome_string: str) -> None:
-        """Writes out a list of genomic sequence strings for comparison to read data."""
+        """Writes out a list of genomic sequence strings for comparison to read data.
+
+        Args:
+            genome_string (str): genomic sequence
+        """
 
         # Grab list of read files in that directory:
         read_files: List[str] = list(self.values.keys())
@@ -258,6 +247,9 @@ class QumaResult:
         """Queue worker for quma functions.
 
         Args:
+            genomic_contents (str): genomic sequence
+            read_contents (str): query sequence
+            position(str): position of reads
 
         Returns:
             Tuple[pd.DataFrame, str]: dataframe of quma results and raw quma results
@@ -275,6 +267,8 @@ class QumaResult:
         """Helper function to run internal QUMA tool.
 
         Args:
+            genomic_contents (str): genome sequence
+            read_contents (str): query sequence
 
         Returns:
             str: output from quma command
@@ -312,8 +306,8 @@ class GenomicPositionData:
             genome_base_lines: List[str] = [s.rstrip("\n") for s in genome_base]
             self.genome_string: str = "".join(map(str, genome_base_lines))
 
-        self._bam_output: Dict[str, BamOutput] = {}
-        self._index_and_fetch()
+        self._bam_output: Dict[str, BamOutput] = self._index_and_fetch()
+
         self.positions: List[str] = self._calculate_positions()
         """List[str]: list of genomic positions in the data."""
 
@@ -335,24 +329,19 @@ class GenomicPositionData:
         self.individual_data: pd.DataFrame = self._return_individual_data()
         """pd.DataFrame: dataframe of individual methylation values."""
 
-    def _index_and_fetch(self) -> None:
+    def _index_and_fetch(self) -> Dict[str, BamOutput]:
         """Wrapper to call processing of each sam file.
 
-        Args:
-            genome_string (str): genomic sequence to align against
-
         Returns:
-            list[str]: list of genomic positions analyzed
-        """
+            Dict[str, BamOutput]: dictionary of bam outputs"""
 
         sam_path: List[Path] = [
             self.config.base_directory / "test" / f for f in self.files_set
         ]
-
+        bam_dict: Dict[str, BamOutput] = {}
         for sams in tqdm(sam_path, desc="Process BAM Files"):
-            self._bam_output[str(sams)] = BamOutput(
-                sams, self.genome_string, self.config
-            )
+            bam_dict[str(sams)] = BamOutput(sams, self.genome_string, self.config)
+        return bam_dict
 
     def _calculate_positions(self) -> List[str]:
         """Return sorted list of all positions analyzed.
@@ -380,7 +369,7 @@ class GenomicPositionData:
         for name, bam_result in tqdm(
             self._bam_output.items(), desc="Process methylation"
         ):
-            cell_line_name: str = name.split("_")[1]
+            cell_line_name: str = Path(name).name.split("_")[1]
 
             read_files: List[str] = [each for each in bam_result.values.values()]
             genomic_files: List[str] = [
@@ -398,7 +387,7 @@ class GenomicPositionData:
         """Save quma results to an excel file.
 
         Args:
-            filename (str, optional): Filename to save to. Defaults to "output.xlsx".
+            filename (str): Filename to save to. Defaults to "output.xlsx".
         """
 
         writer: pd.ExcelWriter = pd.ExcelWriter(
@@ -411,14 +400,25 @@ class GenomicPositionData:
         writer.save()
 
     def save_pickle(self, filename: str) -> None:
-        """Save GenomicPositionData object as a pickled file."""
+        """Save GenomicPositionData object as a pickled file.
+
+        Args:
+            filename (str): filename to save pickle
+        """
 
         with open(filename, "wb") as output_file:
             pickle.dump(self, output_file)
 
     @staticmethod
     def from_pickle(filename: str) -> GPD:
-        """Read pickled GenomicPositionData back to an object."""
+        """Read pickled GenomicPositionData back to an object.
+
+        Args:
+            filename (str): filename to read pickle
+
+        Returns:
+            GPD: GenomicPositionData object
+        """
 
         with open(filename, "rb") as input_file:
             data = pickle.load(input_file)  # nosec
@@ -434,8 +434,9 @@ class GenomicPositionData:
 
         # Gives the means of each positions-- NOT the mean of the entire dataframe!
         working_df: pd.DataFrame = pd.DataFrame()
+        pos_dict: Dict[str, str] = {each: "" for each in self.positions}
+        working_df = working_df.assign(**pos_dict)
         for pos in self.positions:
-            working_df[pos] = ""
             for key in self.quma_results.keys():
                 values_list: List[float] = self._return_read_values(pos, key)
                 if values_list:
@@ -458,8 +459,9 @@ class GenomicPositionData:
 
         # Gives the modes of each positions-- NOT the mode of the entire dataframe!
         working_df: pd.DataFrame = pd.DataFrame()
+        pos_dict: Dict[str, str] = {each: "" for each in self.positions}
+        working_df = working_df.assign(**pos_dict)
         for pos in self.positions:
-            working_df[pos] = ""
             for key in self.quma_results.keys():
                 values_list: List[float] = self._return_read_values(pos, key)
 
@@ -481,9 +483,10 @@ class GenomicPositionData:
             pd.DataFrame: methylation values for each position in each cell line
         """
 
-        working_df: pd.DataFrame = pd.DataFrame()
+        working_df: pd.DataFrame = pd.DataFrame(
+            index=list(self.quma_results.keys()), columns=self.positions
+        )
         for pos in self.positions:
-            working_df[pos] = ""  # Create position column in dataframe
             for key in self.quma_results.keys():
                 values_list: List[float] = self._return_read_values(pos, key)
                 if values_list:
@@ -515,7 +518,7 @@ class GenomicPositionData:
 
         bad_values: List[str] = ["N", "F"]  # for interpreting quma returns
         min_num_meth_sites: int = (
-            min_sites  # Only include is read has minimum methylation sites
+            min_sites  # Only include if read has minimum methylation sites
         )
 
         values_list: List[float] = []
@@ -717,6 +720,9 @@ class GenomicPositionData:
             cell_lines (Optional[List[str]]): set of cell lines to analyze,
             defaults to all cell lines.
             data_type (str): type of data to plot. Can to 'means', 'modes', or 'diffs'
+
+        Raises:
+            ValueError: invalid data type
         """
 
         title_type: str
@@ -851,7 +857,7 @@ class GenomicPositionData:
             raw_list (pd.Series): list of fractional methylation levels per read.
 
         Returns:
-            AD_stats[bool, float, List[np.ndarray]]: is the data significantly
+            AD_stats: is the data significantly
                 allelic (bool), A-D statistic (float), critical values (list)
         """
 
@@ -912,7 +918,7 @@ def make_list_of_bam_files(config: Config) -> List[str]:
     """Check analysis directory for all valid .bam files.
 
     Args:
-        Config: pyllelic configuration options.
+        config (Config): pyllelic configuration options.
 
     Returns:
         list[str]: list of files
